import { Component, ElementRef, EventEmitter, forwardRef, Inject, Injector, Optional, ViewChild, ViewEncapsulation } from '@angular/core';
import { OFormComponent } from 'ontimize-web-ngx';
import { DFComponents } from './components/components';
import { ODynamicFormEvents } from './o-dynamic-form.events';
import { ODFElementComponent } from './o-dynamic-form-element.component';
var ODFComponentComponent = (function () {
    function ODFComponentComponent(oForm, elRef, injector, events) {
        this.oForm = oForm;
        this.elRef = elRef;
        this.injector = injector;
        this.events = events;
        this.show = true;
        this.components = [];
        this.editMode = false;
        this.render = new EventEmitter();
        this.isDragEnabled = false;
    }
    ODFComponentComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.addComponent();
        this.events.onChange.subscribe(function () { return _this.checkConditions(); });
    };
    ODFComponentComponent.prototype.checkConditions = function () {
        this.show = true;
    };
    ODFComponentComponent.prototype.addComponent = function () {
        var component = DFComponents.createComponent(this.component, this.events, this.injector);
        if (component) {
            component.index = this.components.length;
            this.components.push(component);
        }
        else {
            this.render.emit(true);
        }
        return component;
    };
    ODFComponentComponent.prototype.removeAt = function (index) {
        this.components.splice(index, 1);
    };
    ODFComponentComponent.prototype.isContainerComponent = function (component) {
        return component.isContainerComponent();
    };
    ODFComponentComponent.prototype.getDraggableData = function () {
        return this.odfElement ? this.odfElement.component : null;
    };
    ODFComponentComponent.decorators = [
        { type: Component, args: [{
                    selector: 'odf-component',
                    template: "\n    <div *ngFor=\"let comp of components; let i = index;\" class=\"odf-element-container\" fxLayout=\"column\"\n      dnd-draggable [dragData]=\"getDraggableData()\" [dragEnabled]=\"isDragEnabled\">\n\n      <div [ngClass]=\"{'component-container': isContainerComponent(comp), 'component': !isContainerComponent(comp)}\">\n\n        <!-- dnd-draggable-handle -->\n        <odf-element-options\n          (mouseenter)=\"isDragEnabled=true\"\n          (mouseleave)=\"isDragEnabled=false\"\n          fxLayout\n          *ngIf=\"editMode\"\n          [component]=\"comp\"\n          [edit-component-settings-emitter]=\"editComponentSettingsEmitter\"\n          [delete-component-emitter]=\"deleteComponentEmitter\">\n        </odf-element-options>\n\n        <odf-element\n          #odfElement\n          fxLayout=\"column\"\n          [component]=\"comp\"\n          [render]=\"render\"\n          [edit-mode]=\"editMode\"\n          [add-component-emitter]=\"addComponentEmitter\"\n          [move-component-emitter]=\"moveComponentEmitter\"\n          [edit-component-settings-emitter]=\"editComponentSettingsEmitter\"\n          [delete-component-emitter]=\"deleteComponentEmitter\">\n        </odf-element>\n\n      </div>\n\n      <o-drop-zone fxFlex\n        *ngIf=\"editMode\"\n        for-children=\"false\"\n        [component]=\"comp\"\n        [add-component-emitter]=\"addComponentEmitter\"\n        [move-component-emitter]=\"moveComponentEmitter\">\n      </o-drop-zone>\n\n    </div>\n \n  ",
                    inputs: [
                        'component',
                        'editMode : edit-mode',
                        'addComponentEmitter : add-component-emitter',
                        'moveComponentEmitter : move-component-emitter',
                        'editComponentSettingsEmitter : edit-component-settings-emitter',
                        'deleteComponentEmitter : delete-component-emitter'
                    ],
                    outputs: [
                        'render'
                    ],
                    encapsulation: ViewEncapsulation.None
                },] },
    ];
    ODFComponentComponent.ctorParameters = function () { return [
        { type: OFormComponent, decorators: [{ type: Optional }, { type: Inject, args: [forwardRef(function () { return OFormComponent; }),] },] },
        { type: ElementRef, },
        { type: Injector, },
        { type: ODynamicFormEvents, },
    ]; };
    ODFComponentComponent.propDecorators = {
        "odfElement": [{ type: ViewChild, args: ['odfElement',] },],
    };
    return ODFComponentComponent;
}());
export { ODFComponentComponent };
//# sourceMappingURL=o-dynamic-form-component.component.js.map