var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ElementRef, EventEmitter, forwardRef, Inject, Injector, Optional, ViewEncapsulation, } from '@angular/core';
import { ActivatedRoute } from '@angular/router';
import { Observable } from 'rxjs/Observable';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import 'rxjs/add/observable/combineLatest';
import { InputConverter, OFormComponent, OFormValue, OntimizeService, SQLTypes, Util, ServiceUtils } from 'ontimize-web-ngx';
import { ODynamicFormEvents } from './o-dynamic-form.events';
var ODynamicFormComponent = (function () {
    function ODynamicFormComponent(actRoute, injector, events, elRef, parentForm) {
        this.actRoute = actRoute;
        this.injector = injector;
        this.events = events;
        this.elRef = elRef;
        this.parentForm = parentForm;
        this.ready = new BehaviorSubject(false);
        this.editMode = false;
        this.keys = '';
        this.parentKeys = '';
        this.columns = '';
        this.queryOnInit = true;
        this.queryOnBind = true;
        this.queryOnRender = true;
        this.registerInParentForm = true;
        this.autoBinding = true;
        this.autoRegistering = true;
        this.innerFormDefinition = null;
        this.keysArray = [];
        this.colsArray = [];
        this._pKeysEquiv = {};
        this.render = new EventEmitter();
        this.submit = new EventEmitter();
        this.change = new EventEmitter();
        this.onAddComponent = new EventEmitter();
        this.onMoveComponent = new EventEmitter();
        this.onEditComponentSettings = new EventEmitter();
        this.onDeleteComponent = new EventEmitter();
        this.onDynamicFormDataLoaded = new EventEmitter();
        this.onFormInitStream = new EventEmitter();
        this.onUrlParamChangedStream = new EventEmitter();
        this.loading = false;
        this.formData = {};
        this.reloadStream = Observable.combineLatest(this.onFormInitStream.asObservable(), this.onUrlParamChangedStream.asObservable());
        var self = this;
        this.reloadStream.subscribe(function (valArr) {
            if (Util.isArray(valArr) && valArr.length === 2) {
                if (self.queryOnInit && valArr[0] === true && valArr[1] === true) {
                    self.doQuery(true);
                }
            }
        });
    }
    ODynamicFormComponent.prototype.ngOnInit = function () {
        if (this.formDefinition) {
            this.ready.next(true);
        }
        this.keysArray = Util.parseArray(this.keys);
        this.colsArray = Util.parseArray(this.columns);
        var pkArray = Util.parseArray(this.parentKeys);
        this._pKeysEquiv = Util.parseParentKeysEquivalences(pkArray);
        this.configureService();
        this.registerFormListeners();
        var self = this;
        this.urlParamSub = this.actRoute.params.subscribe(function (params) {
            self.urlParams = params;
            if (self.urlParams && Object.keys(self.urlParams).length > 0) {
                self.onUrlParamChangedStream.emit(true);
            }
        });
        if (this.parentForm) {
            if (self.queryOnBind) {
                this.onFormDataSubscribe = this.parentForm.onFormDataLoaded.subscribe(function (data) {
                    var filter = ServiceUtils.getParentItemFromForm(undefined, self._pKeysEquiv, self.parentForm);
                    self.queryData(filter);
                });
            }
        }
    };
    ODynamicFormComponent.prototype.getSQLType = function () {
        return SQLTypes.OTHER;
    };
    ODynamicFormComponent.prototype.getAttribute = function () {
        if (this.oattr) {
            return this.oattr;
        }
        else if (this.elRef && this.elRef.nativeElement.attributes['attr']) {
            return this.elRef.nativeElement.attributes['attr'].value;
        }
    };
    ODynamicFormComponent.prototype.ngAfterViewInit = function () {
        this.onFormInitStream.emit(true);
    };
    ODynamicFormComponent.prototype.configureService = function () {
        var loadingService = OntimizeService;
        if (this.serviceType) {
            loadingService = this.serviceType;
        }
        try {
            this.dataService = this.injector.get(loadingService);
            if (Util.isDataService(this.dataService)) {
                var serviceCfg = this.dataService.getDefaultServiceConfiguration(this.service);
                if (this.entity) {
                    serviceCfg['entity'] = this.entity;
                }
                this.dataService.configureService(serviceCfg);
            }
        }
        catch (e) {
            console.error(e);
        }
    };
    ODynamicFormComponent.prototype.registerFormListeners = function () {
        if (this.parentForm && this.registerInParentForm) {
            this.parentForm.registerFormComponent(this);
            this.parentForm.registerDynamicFormComponent(this);
            this.parentForm.registerSQLTypeFormComponent(this);
        }
    };
    ODynamicFormComponent.prototype.unregisterFormListeners = function () {
        if (this.parentForm && this.registerInParentForm) {
            this.parentForm.unregisterFormComponent(this);
            this.parentForm.unregisterDynamicFormComponent(this);
            this.parentForm.unregisterSQLTypeFormComponent(this);
        }
    };
    ODynamicFormComponent.prototype.setValue = function (val) {
        this.formDefinition = val;
    };
    Object.defineProperty(ODynamicFormComponent.prototype, "data", {
        set: function (value) {
            var formDef = undefined;
            if (value instanceof OFormValue) {
                formDef = value.value;
            }
            if (!formDef) {
                formDef = {};
            }
            this.formDefinition = formDef;
        },
        enumerable: true,
        configurable: true
    });
    ODynamicFormComponent.prototype.isAutomaticBinding = function () {
        return this.autoBinding;
    };
    ODynamicFormComponent.prototype.isAutomaticRegistering = function () {
        return this.autoRegistering;
    };
    ODynamicFormComponent.prototype.getCurrentKeysValues = function () {
        var _this = this;
        var filter = {};
        if (this.urlParams && this.keysArray) {
            this.keysArray.map(function (key) {
                if (_this.urlParams[key]) {
                    filter[key] = _this.urlParams[key];
                }
            });
        }
        var keys = Object.keys(this._pKeysEquiv);
        if (this.urlParams && keys && keys.length > 0) {
            keys.forEach(function (item) {
                var urlVal = _this.urlParams[_this._pKeysEquiv[item]];
                if (urlVal) {
                    filter[item] = urlVal;
                }
            });
        }
        return filter;
    };
    ODynamicFormComponent.prototype.doQuery = function (useFilter) {
        if (useFilter === void 0) { useFilter = false; }
        var filter = {};
        if (useFilter) {
            filter = this.getCurrentKeysValues();
        }
        this.queryData(filter);
    };
    ODynamicFormComponent.prototype.getAttributesToQuery = function () {
        var attributes = [];
        if (this.keysArray && this.keysArray.length > 0) {
            attributes.push.apply(attributes, this.keysArray);
        }
        if (this.oattr) {
            attributes.push(this.oattr);
        }
        return attributes;
    };
    ODynamicFormComponent.prototype._setData = function (data) {
        if (Util.isArray(data) && data.length === 1) {
            this.formData = data[0];
            this._emitData(this.formData);
        }
        else if (Util.isObject(data)) {
            this.formData = data;
            this._emitData(this.formData);
        }
        else {
            console.warn('DynamicForm has received not supported service data. Supported data are Array or Object');
        }
        if (this.oattr) {
            var dynamicData = this.formData[this.oattr];
            this.formDefinition = dynamicData;
        }
    };
    ODynamicFormComponent.prototype._emitData = function (data) {
        this.onDynamicFormDataLoaded.emit(data);
    };
    ODynamicFormComponent.prototype.queryData = function (filter) {
        if (!Util.isDefined(filter) || (Util.isObject(filter) && Object.keys(filter).length === 0)) {
            return;
        }
        var self = this;
        var loader = self.load();
        if (this.dataService === undefined) {
            console.warn('No service configured! aborting query');
            return;
        }
        this.dataService.query(filter, this.getAttributesToQuery(), this.entity).subscribe(function (resp) {
            loader.unsubscribe();
            if (resp.code === 0) {
                self._setData(resp.data);
            }
            else {
                console.log('error ');
            }
        }, function (err) {
            console.log(err);
            loader.unsubscribe();
        });
    };
    Object.defineProperty(ODynamicFormComponent.prototype, "areEmptyComponents", {
        get: function () {
            if (this.formDefinition && this.formDefinition.components) {
                return this.formDefinition.components.length === 0;
            }
            return true;
        },
        enumerable: true,
        configurable: true
    });
    ODynamicFormComponent.prototype.ngOnDestroy = function () {
        if (this.urlParamSub) {
            this.urlParamSub.unsubscribe();
        }
        this.unregisterFormListeners();
        if (this.onFormDataSubscribe) {
            this.onFormDataSubscribe.unsubscribe();
        }
    };
    ODynamicFormComponent.prototype.onComponentRendered = function () {
        if (this.render) {
            this.render.emit(this.queryOnRender);
        }
    };
    ODynamicFormComponent.prototype.load = function () {
        var self = this;
        var loadObservable = new Observable(function (observer) {
            var timer = window.setTimeout(function () {
                observer.next(true);
            }, 250);
            return function () {
                window.clearTimeout(timer);
                self.loading = false;
            };
        });
        var subscription = loadObservable.subscribe(function (val) {
            self.loading = val;
        });
        return subscription;
    };
    Object.defineProperty(ODynamicFormComponent.prototype, "formDefinition", {
        get: function () {
            return this.innerFormDefinition;
        },
        set: function (definition) {
            var definitionJSON = definition;
            if (definition && typeof definition === 'string') {
                try {
                    definitionJSON = JSON.parse(definition);
                }
                catch (e) {
                    console.error('set formDefinition error');
                }
            }
            this.innerFormDefinition = definitionJSON;
        },
        enumerable: true,
        configurable: true
    });
    ODynamicFormComponent.prototype.getControl = function () {
        return undefined;
    };
    ODynamicFormComponent.prototype.getFormControl = function () {
        return undefined;
    };
    ODynamicFormComponent.prototype.hasError = function (error) {
        return false;
    };
    ODynamicFormComponent.decorators = [
        { type: Component, args: [{
                    selector: 'o-dynamic-form',
                    template: "\n    <div fxFlex fxFlexFill fxLayout=\"column\">\n\n      <o-drop-zone fxFlex\n        id=\"main\"\n        *ngIf=\"editMode\"\n        [add-component-emitter]=\"onAddComponent\"\n        [move-component-emitter]=\"onMoveComponent\">\n      </o-drop-zone>\n\n      <ng-template ngFor let-component [ngForOf]=\"formDefinition?.components\">\n        <odf-component fxFlex layout-padding\n          *ngIf=\"ready\"\n          [component]=\"component\"\n          [edit-mode]=\"editMode\"\n          (render)=\"onComponentRendered()\"\n          (change)=\"$event.stopPropagation()\"\n          [add-component-emitter]=\"onAddComponent\"\n          [move-component-emitter]=\"onMoveComponent\"\n          [edit-component-settings-emitter]=\"onEditComponentSettings\"\n          [delete-component-emitter]=\"onDeleteComponent\">\n        </odf-component>\n      </ng-template>\n\n    </div>\n  ",
                    styles: ["\n    .odf-element-options-container{z-index:1;background:#DADADA;cursor:pointer}.odf-element-options-container .odf-element-options-attr{display:inline-flex;align-items:center;padding-left:8px}.odf-element-options-container mat-icon{font-size:14px;cursor:pointer;padding:5px;box-sizing:border-box}odf-element o-column .container-content,odf-element o-row .container-content{padding:0}.odf-element-container{position:relative;cursor:inherit !important}.odf-element-container .component-container{border:1px solid #DADADA}.odf-element-container .component-container o-column.margin-top-bottom,.odf-element-container .component-container o-row.margin-top-bottom{margin:0;padding:8px 0px}\n  "],
                    inputs: [
                        'oattr : attr',
                        'formDefinition: form-definition',
                        'editMode : edit-mode',
                        'entity',
                        'keys',
                        'parentKeys: parent-keys',
                        'columns',
                        'service',
                        'serviceType : service-type',
                        'queryOnInit : query-on-init',
                        'queryOnBind : query-on-bind',
                        'queryOnRender : query-on-render',
                        'registerInParentForm : register-in-parent-form',
                        'autoBinding: automatic-binding',
                        'autoRegistering: automatic-registering'
                    ],
                    outputs: [
                        'render',
                        'submit',
                        'change',
                        'onAddComponent',
                        'onMoveComponent',
                        'onEditComponentSettings',
                        'onDeleteComponent',
                        'onDynamicFormDataLoaded'
                    ],
                    encapsulation: ViewEncapsulation.None
                },] },
    ];
    ODynamicFormComponent.ctorParameters = function () { return [
        { type: ActivatedRoute, },
        { type: Injector, },
        { type: ODynamicFormEvents, },
        { type: ElementRef, },
        { type: OFormComponent, decorators: [{ type: Optional }, { type: Inject, args: [forwardRef(function () { return OFormComponent; }),] },] },
    ]; };
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "editMode", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "queryOnInit", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "queryOnBind", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "queryOnRender", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "registerInParentForm", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "autoBinding", void 0);
    __decorate([
        InputConverter(),
        __metadata("design:type", Boolean)
    ], ODynamicFormComponent.prototype, "autoRegistering", void 0);
    return ODynamicFormComponent;
}());
export { ODynamicFormComponent };
//# sourceMappingURL=o-dynamic-form.component.js.map